from cattorch.util.instruction.instruction import Instruction
from cattorch.util.instruction.elu import ELUInstruction
from cattorch.util.instruction.gelu import GELUInstruction
from cattorch.util.instruction.layernorm import LayerNormInstruction
from cattorch.util.instruction.leaky_relu import LeakyReLUInstruction
from cattorch.util.instruction.matmul import MatMulInstruction
from cattorch.util.instruction.relu import ReLUInstruction
from cattorch.util.instruction.scalar_divide import ScalarDivideInstruction
from cattorch.util.instruction.scalar_multiply import ScalarMultiplyInstruction
from cattorch.util.instruction.sigmoid import SigmoidInstruction
from cattorch.util.instruction.silu import SiLUInstruction
from cattorch.util.instruction.softmax import SoftmaxInstruction
from cattorch.util.instruction.tanh import TanhInstruction
from cattorch.util.instruction.tensor_add import TensorAddInstruction
from cattorch.util.instruction.transpose import TransposeInstruction
